@@ -189,12 +189,11 @@ class PyFiBot(irc.IRCClient, CoreCommands):
     lineRate = 0.5
     hasQuit = False
 
-    CMDCHAR = "."
-
     # Rolling ping time average
     pingAve = 0.0
 
-    def __init__(self, network):
+    def __init__(self, config, network):
+        self.cmdchar = config.get('cmdchar', '.')
         self.network = network
         self.nickname = self.network.nickname
         self.lineRate = self.network.linerate
@@ -318,20 +317,20 @@ class PyFiBot(irc.IRCClient, CoreCommands):
         nickl = len(lnick)
         if channel == lnick:
             # Turn private queries into a format we can understand
-            if not msg.startswith(self.CMDCHAR):
-                msg = self.CMDCHAR + msg
+            if not msg.startswith(self.cmdchar):
+                msg = self.cmdchar + msg
             elif lmsg.startswith(lnick):
-                msg = self.CMDCHAR + msg[nickl:].lstrip()
+                msg = self.cmdchar + msg[nickl:].lstrip()
             elif lmsg.startswith(lnick) and len(lmsg) > nickl and lmsg[nickl] in string.punctuation:
-                msg = self.CMDCHAR + msg[nickl + 1:].lstrip()
+                msg = self.cmdchar + msg[nickl + 1:].lstrip()
         else:
-            # Turn 'nick:' prefixes into self.CMDCHAR prefixes
+            # Turn 'nick:' prefixes into self.cmdchar prefixes
             if lmsg.startswith(lnick) and len(lmsg) > nickl and lmsg[nickl] in string.punctuation:
-                msg = self.CMDCHAR + msg[len(self.nickname) + 1:].lstrip()
+                msg = self.cmdchar + msg[len(self.nickname) + 1:].lstrip()
         reply = (channel == lnick) and user or channel
 
-        if msg.startswith(self.CMDCHAR):
-            cmnd = msg[len(self.CMDCHAR):]
+        if msg.startswith(self.cmdchar):
+            cmnd = msg[len(self.cmdchar):]
             self._command(user, reply, cmnd)
 
         # Run privmsg handlers
